/*
 * This file is part of be_cbo.
 * Copyright 2009-2010 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#ifndef CONVERT_H
#define CONVERT_H

#include <LinearMath/btQuaternion.h>
#include <LinearMath/btVector3.h>
#include <osg/Quat>
#include <osg/Vec3>

/**
 * Class with functions converting between Bullet and OpenSceneGraph objects.
 *
 * @brief Conversion between Bullet and OpenSceneGraph
 * @author Benjamin Eikel
 * @date 2009-04-14
 */
class Convert {
	public:
		/**
		 * Convert a Bullet vector to an OpenSceneGraph vector.
		 *
		 * @param vec Bullet vector
		 * @return OpenSceneGraph vector
		 */
		inline static osg::Vec3 fromBtVec(const btVector3 & vec) {
			return osg::Vec3(vec.x(), vec.y(), vec.z());
		}

		/**
		 * Convert a Bullet quaternion to an OpenSceneGraph quaternion.
		 *
		 * @param vec Bullet quaternion
		 * @return OpenSceneGraph quaternion
		 */
		inline static osg::Quat fromBtQuat(const btQuaternion & quat) {
			return osg::Quat(quat.x(), quat.y(), quat.z(), quat.w());
		}

		/**
		 * Convert an OpenSceneGraph vector to a Bullet vector.
		 *
		 * @param vec OpenSceneGraph vector
		 * @return Bullet vector
		 */
		inline static btVector3 toBtVec(const osg::Vec3 & vec) {
			return btVector3(vec.x(), vec.y(), vec.z());
		}

		/**
		 * Convert an OpenSceneGraph quaternion to a Bullet quaternion.
		 *
		 * @param vec OpenSceneGraph quaternion
		 * @return Bullet quaternion
		 */
		inline static btQuaternion toBtQuat(const osg::Quat & quat) {
			return btQuaternion(quat.x(), quat.y(), quat.z(), quat.w());
		}
}; 

#endif // CONVERT_H
