/*
 * This file is part of be_cbo.
 * Copyright 2010 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#include "DebugDrawer.h"
#include <GL/gl.h>
#include <iostream>
#include <stdint.h>

DebugDrawer::DebugDrawer() : m_debugMode(0) {
}

void DebugDrawer::drawLine(	const btVector3 & from,
							const btVector3 & to,
							const btVector3 & fromColor,
							const btVector3 & toColor) {
	glBegin(GL_LINES);
	glColor3fv(fromColor.m_floats);
	glVertex3fv(from.m_floats);
	glColor3fv(toColor.m_floats);
	glVertex3fv(to.m_floats);
	glEnd();
}

void DebugDrawer::drawLine(	const btVector3 & from,
							const btVector3 & to,
							const btVector3 & color) {
	drawLine(from,to,color,color);
}

void DebugDrawer::drawSphere(	const btVector3 & p,
								btScalar radius,
								const btVector3 & color) {
	// This code is taken from the Bullet OpenGL demo.
	glColor3fv(color.m_floats);
	glPushMatrix();
	glTranslatef(p.x(), p.y(), p.z());

	const uint_fast8_t lats = 5;
	const uint_fast8_t longs = 5;

	for(uint_fast8_t i = 0; i <= lats; ++i) {
		btScalar lat0 = SIMD_PI * (-btScalar(0.5) + (btScalar) (i - 1) / lats);
		btScalar z0 = radius * sin(lat0);
		btScalar zr0 = radius * cos(lat0);

		btScalar lat1 = SIMD_PI * (-btScalar(0.5) + (btScalar) i / lats);
		btScalar z1 = radius * sin(lat1);
		btScalar zr1 = radius * cos(lat1);

		glBegin(GL_QUAD_STRIP);
		for(uint_fast8_t j = 0; j <= longs; ++j) {
			btScalar lng = 2 * SIMD_PI * (btScalar) (j - 1) / longs;
			btScalar x = cos(lng);
			btScalar y = sin(lng);

			glNormal3f(x * zr0, y * zr0, z0);
			glVertex3f(x * zr0, y * zr0, z0);
			glNormal3f(x * zr1, y * zr1, z1);
			glVertex3f(x * zr1, y * zr1, z1);
		}
		glEnd();
	}

	glPopMatrix();
}

void DebugDrawer::drawBox(	const btVector3 & boxMin,
							const btVector3 & boxMax,
							const btVector3 & color,
							btScalar alpha) {
	static const unsigned int index[] = {
		1,3,2,0,
		5,7,3,1,
		4,6,7,5,
		0,2,6,4,
		7,6,2,3,
		4,5,1,0
	};
	float corners[8][3] = {{boxMax.x(), boxMax.y(), boxMax.z()},
					{boxMin.x(), boxMax.y(), boxMax.z()},
					{boxMax.x(), boxMin.y(), boxMax.z()},
					{boxMin.x(), boxMin.y(), boxMax.z()},
					{boxMax.x(), boxMax.y(), boxMin.z()},
					{boxMin.x(), boxMax.y(), boxMin.z()},
					{boxMax.x(), boxMin.y(), boxMin.z()},
					{boxMin.x(), boxMin.y(), boxMin.z()}};

	glBegin(GL_QUADS);
	glColor4f(color.x(), color.y(), color.z(), alpha);
	for(uint_fast8_t i = 0; i < 24; ++i) {
		glVertex3fv(corners[index[i]]);
	}
	glEnd();
}

void DebugDrawer::drawTriangle(	const btVector3 & a,
								const btVector3 & b,
								const btVector3 & c,
								const btVector3 & color,
								btScalar alpha) {
	const btVector3	n = btCross(b - a, c - a).normalized();
	glBegin(GL_TRIANGLES);
	glColor4f(color.x(), color.y(), color.z(), alpha);
	glNormal3fv(n.m_floats);
	glVertex3fv(a.m_floats);
	glVertex3fv(b.m_floats);
	glVertex3fv(c.m_floats);
	glEnd();
}

void DebugDrawer::draw3dText(	const btVector3 & location,
								const char * textString) {
	//glRasterPos3fv(location.m_floats);
}

void DebugDrawer::reportErrorWarning(const char * warningString) {
	std::cout << "DebugDrawer: " << warningString << std::endl;
}

void DebugDrawer::drawContactPoint(	const btVector3 & pointOnB,
									const btVector3 & normalOnB,
									btScalar distance,
									int /*lifeTime*/,
									const btVector3 & color) {
	btVector3 second = pointOnB + normalOnB * distance;
	
	glBegin(GL_LINES);
	glColor3fv(color.m_floats);
	glVertex3fv(pointOnB.m_floats);
	glVertex3fv(second.m_floats);
	glEnd();
}
