/*
 * This file is part of be_cbo.
 * Copyright 2010 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#ifndef DEBUGDRAWER_H
#define DEBUGDRAWER_H

#include <btBulletDynamicsCommon.h>
#include <LinearMath/btIDebugDraw.h>
#include <osg/Camera>
#include <osg/RenderInfo>

/**
 * Implementation of the Bullet DebugDraw interface using OpenGL.
 *
 * @brief OpenGL debug drawer
 * @author Benjamin Eikel
 * @date 2010-02-21
 */
class DebugDrawer : public btIDebugDraw {
	public:
		DebugDrawer();

		virtual void drawLine(const btVector3 & from,const btVector3 & to,const btVector3 & fromColor, const btVector3 & toColor);
		virtual void drawLine(const btVector3 & from,const btVector3 & to,const btVector3 & color);
		virtual void drawSphere(const btVector3 & p, btScalar radius, const btVector3 & color);
		virtual void drawBox(const btVector3 & boxMin, const btVector3 & boxMax, const btVector3 & color, btScalar alpha);
		virtual void drawTriangle(const btVector3& a, const btVector3 & b, const btVector3 & c, const btVector3 & color, btScalar alpha);
		virtual void drawContactPoint(const btVector3 & PointOnB, const btVector3 & normalOnB, btScalar distance, int lifeTime, const btVector3 & color);
		virtual void reportErrorWarning(const char * warningString);
		virtual void draw3dText(const btVector3 & location, const char * textString);

		virtual void setDebugMode(int debugMode) {
			m_debugMode = debugMode;
		}

		virtual int getDebugMode() const {
			return m_debugMode;
		}

	private:
		int m_debugMode;
};

/**
 * Callback functor which can be used for Bullet debug rendering as post draw
 * callback of an OpenSceneGraph camera.
 *
 * @brief Functor for debug drawing
 * @author Benjamin Eikel
 * @date 2010-02-21
 */
class DebugCallback 
		: public osg::Camera::Camera::DrawCallback::DrawCallback {
	public:
		DebugCallback(btDiscreteDynamicsWorld * debugWorld)
				: m_debugWorld(debugWorld) {
		}

		virtual void operator()(osg::RenderInfo & renderInfo) const {
			// Display all debug output even if it is hidden.
			glClear(GL_DEPTH_BUFFER_BIT);
			// Apply the current camera matrix.
			glPushMatrix();
			glLoadMatrixd(renderInfo.getCurrentCamera()->getViewMatrix().ptr());
			m_debugWorld->debugDrawWorld();
			glPopMatrix();
		}
	private:
		btDiscreteDynamicsWorld * m_debugWorld;
};

#endif // DEBUGDRAWER_H
