/*
 * This file is part of be_cbo.
 * Copyright 2009-2010 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#ifndef DELETESELECTEDNODESVISITOR_H
#define DELETESELECTEDNODESVISITOR_H

#include "GeodeData.h"
#include "TransformData.h"
#include <osg/NodeVisitor>
#include <osgFX/Scribe>
#include <list>

/**
 * Traversal visitor for removing nodes selected by an osgFX::Scribe from the 
 * scene graph.
 *
 * @brief Deletion of selected nodes
 * @author Benjamin Eikel
 * @date 2009-04-15
 * @see OpenSceneGraph example osgkeyboardmouse
 */
class DeleteSelectedNodesVisitor : public osg::NodeVisitor {
	public:
		/**
		 * Standard constructor setting mode to doing a full traversal.
		 */
		DeleteSelectedNodesVisitor() : osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN) {
		}
		
		/**
		 * Check if the given node is of typ osgFX::Scribe. If yes, add it to 
		 * the list of selected nodes.
		 *
		 * @param node Current node of traversal.
		 */
		virtual void apply(osg::Node & node) {
			osgFX::Scribe * scribe = dynamic_cast<osgFX::Scribe *>(&node);
			if(scribe) {
				m_selectedNodes.push_back(scribe);
			} else {
				traverse(node);
			}
		}

		/**
		 * Remove all selected nodes found during the last tree traversal.
		 */
		void pruneSelectedNodes() {
			for(SelectedNodes::iterator itr = m_selectedNodes.begin(); itr != m_selectedNodes.end(); ++itr) {
				pruneNode(itr->get());
			}
		}

	private:
		/**
		 * Remove the given node from the scene graph. If it is a child of a 
		 * physics node (node has TransformData or GeodeData as user data), 
		 * remove the physics node too.
		 *
		 * @param node Node to delete from the scene graph.
		 */
		void pruneNode(osg::Node * node) {
			osg::Node::ParentList parents = node->getParents();
			for(osg::Node::ParentList::iterator pitr = parents.begin(); pitr != parents.end(); ++pitr) {
				osg::Group * parent = *pitr;
				// Check if we have a child of a physics node.
				if(	dynamic_cast<TransformData *>(parent->getUserData()) != NULL ||
					dynamic_cast<GeodeData *>(parent->getUserData()) != NULL) {
					pruneNode(parent);
				} else {
					parent->removeChild(node);
				}
			}
		}

		/**
		 * List of pointers to osgFX::Scribe nodes.
		 */
		typedef std::list<osg::ref_ptr<osgFX::Scribe> > SelectedNodes;

		/**
		 * Storage for selected nodes found during scene graph traversal.
		 */
		SelectedNodes m_selectedNodes;
};

#endif // DELETESELECTEDNODESVISITOR_H
