/*
 * This file is part of be_cbo.
 * Copyright 2009-2010 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#ifndef GEODEDATA_H
#define GEODEDATA_H

#include "Shape.h"
#include <BulletDynamics/Dynamics/btDiscreteDynamicsWorld.h>
#include <BulletDynamics/Dynamics/btRigidBody.h>

/**
 * Class for storing a btRigidBody and a Shape inside an osg::Geode. Intented 
 * to be set as user data of an osg::Geode.
 *
 * @brief Storage for shape and rigid body data.
 * @author Benjamin Eikel
 * @date 2009-04-14
 */
class GeodeData : public osg::Referenced {
	public:
		/**
		 * Create a new physics object of type btRigidBody and add it to the 
		 * given world.
		 *
		 * @param world Physics world.
		 * @param constructionInfo Information for creation of the physics 
		 * object.
		 * @param shape Only stored here for memory management. Is going to be
		 * destroyed when this object is destroyed.
		 */
		GeodeData(	btDiscreteDynamicsWorld * world,
					const btRigidBody::btRigidBodyConstructionInfo & constructionInfo,
					Shape * shape) : 
						m_world(world),
						m_rigidBody(new btRigidBody(constructionInfo)), 
						m_shape(shape) {
			m_world->addRigidBody(m_rigidBody);
		}
	
		/**
		 * Remove the physics object and free the shape.
		 */
		virtual ~GeodeData() {
			m_world->removeRigidBody(m_rigidBody);
			delete m_rigidBody;
			delete m_shape;
		}

		/**
		 * Access to the physics object associated with the osg::Geode.
		 *
		 * @return Bullet physics object.
		 */
		btRigidBody * getRigidBody() {
			return m_rigidBody;
		}

	protected:
		/**
		 * Pointer to the world the rigid body is part of.
		 */
		btDynamicsWorld * m_world;

		/**
		 * Pointer to the physics object associated with the osg::Geode.
		 */
		btRigidBody * m_rigidBody;

		/**
		 * The shape is only stored here because it should be destroyed when 
		 * the osg::Geode gets destroyed.
		 */
		Shape * m_shape;
};

#endif // GEODEDATA_H
