/*
 * This file is part of be_cbo.
 * Copyright 2009-2010 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#include "Shape.h"

#include <BulletCollision/CollisionShapes/btBoxShape.h>
#include <BulletCollision/CollisionShapes/btBvhTriangleMeshShape.h>
#include <BulletCollision/CollisionShapes/btSphereShape.h>
#include <BulletCollision/CollisionShapes/btStaticPlaneShape.h>
#include <BulletCollision/CollisionShapes/btTriangleIndexVertexArray.h>
#include <osg/Drawable>
#include <osg/Geode>
#include <osg/TriangleFunctor>
#include <osgDB/ReadFile>

Shape::Shape(btCollisionShape * collisionShape, osg::Shape * osgShape)
	: m_collisionShape(collisionShape), m_osgShape(osgShape) {
}

Shape::~Shape() {
	delete m_collisionShape;
	m_osgShape = NULL;
}

btCollisionShape * Shape::getCollisionShape() {
	return m_collisionShape;
}

const btCollisionShape * Shape::getCollisionShape() const {
	return m_collisionShape;
}

osg::Shape * Shape::getOsgShape() {
	return m_osgShape;
}

const osg::Shape * Shape::getOsgShape() const {
	return m_osgShape;
}

InfinitePlane::InfinitePlane() 
	: Shape(
		new btStaticPlaneShape(btVector3(0.0f, 0.0f, 1.0f), 0.0f), 
		new osg::InfinitePlane()
	) {
}

Box::Box(btVector3 size) 
	: Shape(
		new btBoxShape(0.5 * size), 
		new osg::Box(osg::Vec3(0.0f, 0.0f, 0.0f), size.x(), size.y(), size.z())
	) {
}

Sphere::Sphere(btScalar radius) 
	: Shape(
		new btSphereShape(radius),
		new osg::Sphere(osg::Vec3(0.0f, 0.0f, 0.0f), radius)
	) {
}

class VerticesCollect {
	public:
		void operator()(
			const osg::Vec3 & v1, 
			const osg::Vec3 & v2, 
			const osg::Vec3 & v3,
			bool /*treatVertexDataAsTemporary*/
		) {
			m_vertices.push_back(v1);
			m_vertices.push_back(v2);
			m_vertices.push_back(v3);
		}

		const std::list<osg::Vec3> & getVertices() const {
			return m_vertices;
		}

	private:
		std::list<osg::Vec3> m_vertices;
};

Mesh::Mesh(const char * path) 
	: Shape(
		NULL, 
		new osg::TriangleMesh
	) {
	osg::Node * node = osgDB::readNodeFile(path);
	if(node == NULL) {
		throw "Error loading mesh file in Mesh::Mesh().";
	}
	
	class CollectGeometryVisitor : public osg::NodeVisitor {
		public:
			
			CollectGeometryVisitor() : osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN) {
			}

			virtual void apply(osg::Node & node) {
				traverse(node);
			}
	
			virtual void apply(osg::Geode & node) {
				// Traverse all Drawables of the Geode.
				osg::Geode::DrawableList drawList = node.getDrawableList();
				for(osg::Geode::DrawableList::iterator it = drawList.begin();
					it != drawList.end();
					++it) {
					// Call the TriangleFunctor to collect the vertices.
					if((*it)->supports(m_triangleFunctor)) {
						(*it)->accept(m_triangleFunctor);
					}
				}
			}

			const std::list<osg::Vec3> & getVertices() const {
				return m_triangleFunctor.getVertices();
			}

		private:
			osg::TriangleFunctor<VerticesCollect> m_triangleFunctor;
	};

	CollectGeometryVisitor cgv;
	node->accept(cgv);

	const std::list<osg::Vec3> & vertices = cgv.getVertices();

	unsigned int numIndices = vertices.size();
	unsigned int * indicesData = new unsigned int[numIndices];
	for(unsigned int i = 0; i < numIndices; ++i) {
		indicesData[i] = i;
	}
	osg::ref_ptr<osg::UIntArray> indices = new osg::UIntArray(numIndices, indicesData);

	osg::TriangleMesh * mesh = dynamic_cast<osg::TriangleMesh *>(m_osgShape.get());
	mesh->setVertices(new osg::Vec3Array(vertices.begin(), vertices.end()));
	mesh->setIndices(indices.get());

	indices = NULL;
	delete [] indicesData;

	btTriangleIndexVertexArray * meshInterface 
		= new btTriangleIndexVertexArray(
			mesh->getIndices()->getNumElements() / 3,
			const_cast<int *>(reinterpret_cast<const int *>(mesh->getIndices()->getDataPointer())),
			3 * sizeof(unsigned int),
			mesh->getVertices()->getNumElements(),
			const_cast<btScalar *>(reinterpret_cast<const btScalar *>(mesh->getVertices()->getDataPointer())),
			3 * sizeof(float) 
		);
	m_collisionShape = new btBvhTriangleMeshShape(meshInterface, true, true);
}
