/*
 * This file is part of be_cbo.
 * Copyright 2009-2010 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#ifndef SHAPE_H
#define SHAPE_H

#include <BulletCollision/CollisionShapes/btCollisionShape.h>
#include <osg/Shape>

/**
 * Class representing one abstract collision object. It contains pointers to a 
 * btCollisionShape and and osg::Shape. This class is intended to be 
 * subclassed.
 *
 * @brief %Shape of a collision object
 * @author Benjamin Eikel
 * @date 2009-04-14
 */
class Shape {
	public:
		/**
		 * Store the OpenSceneGraph and Bullet shapes.
		 *
		 * @param collisionShape Bullet shape
		 * @param osgShape OpenSceneGraph shape
		 */
		Shape(btCollisionShape * collisionShape, osg::Shape * osgShape);

		/**
		 * Free the stored objects.
		 */
		~Shape();

		/**
		 * Access to the Bullet shape.
		 *
		 * @return Bullet shape
		 */
		btCollisionShape * getCollisionShape();

		/**
		 * Constant access to the Bullet shape.
		 *
		 * @return Bullet shape
		 */
		const btCollisionShape * getCollisionShape() const;

		/**
		 * Access to the OpenSceneGraph shape.
		 *
		 * @return OpenSceneGraph shape
		 */
		osg::Shape * getOsgShape();

		/**
		 * Constant access to the OpenSceneGraph shape.
		 *
		 * @return OpenSceneGraph shape
		 */
		const osg::Shape * getOsgShape() const;

	protected:
		/**
		 * Pointer to the Bullet shape.
		 */
		btCollisionShape * m_collisionShape;

		/**
		 * Pointer to the OpenSceneGraph shape.
		 */
		osg::ref_ptr<osg::Shape> m_osgShape;
};

/**
 * Special shape reprensenting an infinite plane.
 *
 * @brief Infinite plane shape
 * @author Benjamin Eikel
 * @date 2010-02-21
 */
class InfinitePlane : public Shape {
	public:
		/**
		 * Create an infinite plane containing the origin.
		 */
		InfinitePlane();
};

/**
 * Special shape reprensenting a box.
 *
 * @brief %Box shape
 * @author Benjamin Eikel
 * @date 2009-04-14
 */
class Box : public Shape {
	public:
		/**
		 * Create a sphere with the given side lengths.
		 *
		 * @param size Side lengths of the box.
		 */
		Box(btVector3 size);
};

/**
 * Special shape reprensenting a sphere.
 *
 * @brief %Sphere shape
 * @author Benjamin Eikel
 * @date 2009-04-14
 */
class Sphere : public Shape {
	public:
		/**
		 * Create a sphere with the given radius.
		 *
		 * @param radius Radius of the sphere.
		 */
		Sphere(btScalar radius);
};

/**
 * Special shape reprensenting a mesh.
 *
 * @brief %Mesh shape
 * @author Benjamin Eikel
 * @date 2009-06-06
 */
class Mesh : public Shape {
	public:
		/**
		 * Load a mesh from the given file.
		 *
		 * @param path Path to a file containing a mesh in a format supported
		 * by OpenSceneGraph.
		 */
		Mesh(const char * path);
};

#endif // SHAPE_H
